config() {
  NEW="$1"
  OLD="$(dirname $NEW)/$(basename $NEW .new)"
  # If there's no config file by that name, mv it over:
  if [ ! -r $OLD ]; then
    mv $NEW $OLD
  elif [ "$(cat $OLD | md5sum)" = "$(cat $NEW | md5sum)" ]; then
    # toss the redundant copy
    rm $NEW
  fi
  # Otherwise, we leave the .new copy for the admin to consider...
}

preserve_perms() {
  NEW="$1"
  OLD="$(dirname $NEW)/$(basename $NEW .new)"
  if [ -e $OLD ]; then
    cp -a $OLD ${NEW}.incoming
    cat $NEW > ${NEW}.incoming
    mv ${NEW}.incoming $NEW
  fi
  config $NEW
}

preserve_perms etc/rc.d/rc.pcscd.new

( cd usr/lib ; rm -rf libpcscspy.so )
( cd usr/lib ; ln -sf libpcscspy.so.0.0.0 libpcscspy.so )
( cd usr/lib ; rm -rf libpcscspy.so.0 )
( cd usr/lib ; ln -sf libpcscspy.so.0.0.0 libpcscspy.so.0 )
( cd usr/lib ; rm -rf libpcsclite.so )
( cd usr/lib ; ln -sf libpcsclite.so.1.0.0 libpcsclite.so )
( cd usr/lib ; rm -rf libpcsclite.so.1 )
( cd usr/lib ; ln -sf libpcsclite.so.1.0.0 libpcsclite.so.1 )
